
/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/


%{
  #include <stdio.h>
  #include <Xm/Xm.h>

  #include "config.h"

  char *temp_str1, *temp_str2, *temp_str3, *temp_str4, *temp_str5;

  void yyerror(char *message) 
  {
    fprintf(stderr,"\nConfig file parse error :\n\n  %s\n\n",message);
    exit(-1);
  }

  char *make_identifier(char *ident)
  {
    char *result;

    result=XtMalloc(strlen(ident)+1);
    strcpy(result,ident);
    return(result);
  }

  char *make_string(char *string)
  {
    char *result;
    int i;

    result=XtMalloc(strlen(string)-1);
    for(i=0;i<strlen(string)-2;i++) {
      result[i]=string[i+1];
    }
    result[i]=(char)0;
    
    return(result);
  }

%}

IDENTIFIER [a-zA-Z_][a-zA-Z_0-9]+
SPACES [ \t]*
INTEGER [-+]?[0-9]+
BOOLEAN "TRUE"|"FALSE"
STRING \"[^\"]*\"

%x var1 var2 var3 format1 format2 description1 description2 pattern1 pattern2 view1 view2 edit1 edit2 pixmap1 pixmap2 menu1 menulist item1 item2 item3 popup1 name1 name2 cmenu1 cmenulist cname1 cname2 citem1 citem2 citem3

%%

 /* comment */

"#"[^\n]*           ;

 /*----------------------- set a variable -------------------*/

"SET"               { 
                      BEGIN(var1);
                    }

<var1>{IDENTIFIER}  { temp_str1=make_identifier(yytext);
                      BEGIN(var2);
	    	    }

<var2>"="           { 
	              BEGIN(var3);
                    }

<var3>{INTEGER}     { AddVarList(temp_str1, atoi(yytext), False, NULL);
	              BEGIN(INITIAL);
                    }

<var3>{BOOLEAN}     { if(strcmp(yytext,"TRUE")==0) 
                        AddVarList(temp_str1, -1, True, NULL);
                      else
                        AddVarList(temp_str1, -1, False, NULL);
	              BEGIN(INITIAL);
                    }

<var3>{STRING}      { char *str;
                    
                      str=make_string(yytext);
                      AddVarList(temp_str1, -1, False, str);
                      BEGIN(INITIAL);
                    }

<var1,var2,var3>[ \t]      ;

<var1>.       { yyerror("identifier expected");
              }

<var2>.       { yyerror("equal sign expected");
              }

<var3>.       { yyerror("variable definition expected");
              }

 /*-------------------- set a fileformat -------------------*/

"FORMAT"      { 
                BEGIN(format1);  
              }

 /* get errors */

<format1,description1,pattern1,view1,edit1,pixmap1>[ \t\n]*  ;
<description2,pattern2,view2,edit2,pixmap2>[ \t]*  ;

 /* format definition */

<format1>{IDENTIFIER} { 
                        BEGIN(description1);
                      }

<description1>"DESCRIPTION" { 
                              BEGIN(description2);
                            }

<description2>{STRING} { temp_str1=make_string(yytext);
                        BEGIN(pattern1);
                      }

<pattern1>"PATTERN" { BEGIN(pattern2); }

<pattern2>{STRING} { temp_str2=make_string(yytext);
                     BEGIN(view1);
                   }

<view1>"VIEW"   { BEGIN(view2); }

<view2>{STRING} { temp_str3=make_string(yytext);
                  BEGIN(edit1);
                }

<edit1>"EDIT" { BEGIN(edit2); }

<edit2>{STRING} { temp_str4=make_string(yytext);
                     BEGIN(pixmap1);
                   }

<pixmap1>"PIXMAP" { BEGIN(pixmap2); }

<pixmap2>{STRING} { char *pixmap;

                     pixmap=make_string(yytext);
		     AddFileFormat(temp_str1, temp_str2, temp_str3, temp_str4, pixmap);
                     BEGIN(format2);
                   }

<format2>"ENDFORMAT" {  
                       BEGIN(INITIAL);   
                     }

<format1>. { yyerror("Format identifier expected"); }
<format2>. { yyerror("ENDFORMAT keyword expected"); }
<description1>. { yyerror("DESCRIPTION keyword expected"); }
<pattern1>. { yyerror("PATTERN keyword expectd"); }
<view1>. { yyerror("VIEW keyword expectd"); }
<edit1>. { yyerror("EDIT keyword expectd"); }
<pixmap1>. { yyerror("PIXMAP keyword expectd"); }
<description2,pattern2,view2,edit2,pixmap2>. { yyerror("STRING expected"); }

 /*------------------------ parse a popmenu -------------------------*/

"MENU" { 
          BEGIN(menu1);
        }

<menu1,name1,menulist>[ \t\n]*  ;
<name2,item1,item2,item3,popup1>[ \t]*  ;

<menu1>{IDENTIFIER} {  temp_str1=make_identifier(yytext); 
                       BEGIN(name1);
                    }

<menu1>. { yyerror("Menu identifier expected"); }

<name1>"NAME" {  
                BEGIN(name2);
              }

<name1>. { yyerror("NAME keyword expected"); }

<name2>{STRING} { char *name;

                  name=make_string(yytext);
                  AddMenu(temp_str1, name);
                  BEGIN(menulist);
		}

<name2>. { yyerror("Menu name string expected"); }

<menulist>"MENUITEM" { 
                       BEGIN(item1);
                     }

<menulist>"POPUP" { 
                    BEGIN(popup1);
                  }

<menulist>"SEPARATOR" { AddMenuSeparator(); 
                        BEGIN(menulist);
                      }

<menulist>"ENDMENU" { 
                      BEGIN(INITIAL);
                    }

<menulist>. { yyerror("Menu definition expected"); }

<item1>{STRING} { temp_str1=make_string(yytext);
                  BEGIN(item2);
                }

<item1>. { yyerror("Menu item name string expected"); }

<item2>"COMMAND" { 
                   BEGIN(item3);
                 }

<item2>. { yyerror("COMMAND keyword expected"); }

<item3>{STRING} { char *command;

                  command=make_string(yytext);
                  AddMenuItem(temp_str1, command);
                  BEGIN(menulist);
                }

<item3>. { yyerror("Command name string expected"); }

<popup1>{IDENTIFIER} {  char *ident;

                        ident=make_identifier(yytext);
                        AddMenuPopup(ident);
                        BEGIN(menulist);
                      }

<popup1>.  { yyerror("Popup identifier expected"); }

 /*---------------------- parse the context menus -------------------------*/

"CMENU" { 
          BEGIN(cmenu1);
        }

<cmenu1,cname1,cmenulist>[ \t\n]*  ;
<cname2,citem1,citem2,citem3>[ \t]*  ;

<cmenu1>{IDENTIFIER} {  temp_str1=make_identifier(yytext); 
                       BEGIN(cname1);
                    }

<cmenu1>. { yyerror("Menu identifier expected"); }

<cname1>"NAME" {  
                BEGIN(cname2);
              }

<cname1>. { yyerror("NAME keyword expected"); }

<cname2>{STRING} { char *name;

                  name=make_string(yytext);
                  AddCMenu(temp_str1, name);
                  BEGIN(cmenulist);
		}

<cname2>. { yyerror("Menu name string expected"); }

<cmenulist>"MENUITEM" { 
                       BEGIN(citem1);
                     }

<cmenulist>"SEPARATOR" { AddCMenuSeparator(); 
                        BEGIN(cmenulist);
                      }

<cmenulist>"ENDCMENU" { 
                      BEGIN(INITIAL);
                    }

<cmenulist>. { yyerror("Menu definition expected"); }

<citem1>{STRING} { temp_str1=make_string(yytext);
                   BEGIN(citem2);
                }

<citem1>. { yyerror("Menu item name string expected"); }

<citem2>"COMMAND" { 
                   BEGIN(citem3);
                 }

<citem2>. { yyerror("COMMAND keyword expected"); }

<citem3>{STRING} { char *command;

                  command=make_string(yytext);
                  AddCMenuItem(temp_str1, command);
                  BEGIN(cmenulist);
                }

<citem3>. { yyerror("Command name string expected"); }

 /*--------------------------- misceallaneous ------------------------*/

[ \t\n]            ;

. { yyerror("not recognized character in config file"); }

%%

int yywrap(void)
{
 return 1;
}










